/*
 * Copyright (c) 2002 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * This file contains Original Code and/or Modifications of Original Code
 * as defined in and that are subject to the Apple Public Source License
 * Version 2.0 (the 'License'). You may not use this file except in
 * compliance with the License. Please obtain a copy of the License at
 * http://www.opensource.apple.com/apsl/ and read it before using this
 * file.
 * 
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
 * Please see the License for the specific language governing rights and
 * limitations under the License.
 * 
 * @APPLE_LICENSE_HEADER_END@
 */

/*!
 * @header DSMutexSemaphore
 * Implementation of the DSMutexSemaphore ( mutually exclusive lock ) class.
 */

#include <errno.h>
#include <sys/time.h>	// for struct timespec and gettimeofday()

#include "DSMutexSemaphore.h"

//--------------------------------------------------------------------------------------------------
//	DSMutexSemaphore class implementation
//--------------------------------------------------------------------------------------------------

DSMutexSemaphore::DSMutexSemaphore ( bool owned )
	: DSSemaphore ( owned ? 0 : 1 ),
		mOwner ( owned ? ::pthread_self() : NULL )
{
	pthread_mutexattr_t mutexType;
	
	pthread_mutexattr_init( &mutexType );
	pthread_mutexattr_settype( &mutexType, PTHREAD_MUTEX_RECURSIVE );
	pthread_mutex_init( &mConditionLock, &mutexType );
	pthread_mutexattr_destroy( &mutexType );
}

DSMutexSemaphore::~DSMutexSemaphore ( void )
{
}

void
DSMutexSemaphore::Signal ( void )
{
	pthread_mutex_unlock ( &mConditionLock );
} // Signal


sInt32 DSMutexSemaphore::Wait ( sInt32 milliSecs )
{
	pthread_mutex_lock ( &mConditionLock );
	return( semNoErr );

} // Wait
