/*
 * Copyright (c) 2002 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * This file contains Original Code and/or Modifications of Original Code
 * as defined in and that are subject to the Apple Public Source License
 * Version 2.0 (the 'License'). You may not use this file except in
 * compliance with the License. Please obtain a copy of the License at
 * http://www.opensource.apple.com/apsl/ and read it before using this
 * file.
 * 
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
 * Please see the License for the specific language governing rights and
 * limitations under the License.
 * 
 * @APPLE_LICENSE_HEADER_END@
 */

/*!
 * @header DirectoryServiceCorePriv
 * Umbrella header file for the private core API header files.
 */

#ifndef __DirectoryServiceCorePriv_h__
#define	__DirectoryServiceCorePriv_h__	1

#include <DirectoryServiceCore/CBuff.h>
#include <DirectoryServiceCore/CDSServerModule.h>
#include <DirectoryServiceCore/CDataBuff.h>
#include <DirectoryServiceCore/CFile.h>
#include <DirectoryServiceCore/CLog.h>
#include <DirectoryServiceCore/COSUtils.h>
#include <DirectoryServiceCore/CRCCalc.h>
#include <DirectoryServiceCore/CString.h>
#include <DirectoryServiceCore/DNSLookups.h>
#include <DirectoryServiceCore/DSCThread.h>
#include <DirectoryServiceCore/DSEventSemaphore.h>
#include <DirectoryServiceCore/DSLThread.h>
#include <DirectoryServiceCore/DSLogException.h>
#include <DirectoryServiceCore/DSMutexSemaphore.h>
#include <DirectoryServiceCore/DSSemaphore.h>
#include <DirectoryServiceCore/DSUtils.h>
#include <DirectoryServiceCore/GetMACAddress.h>
#include <DirectoryServiceCore/PrivateTypes.h>
#include <DirectoryServiceCore/ServerModule.h>
#include <DirectoryServiceCore/ServerModuleLib.h>
#include <DirectoryServiceCore/SharedConsts.h>

#endif  // __DirectoryServiceCorePriv_h__
